@@ -23,6 +23,7 @@ import (
 	"path"
 	"path/filepath"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/bep/logg"
@@ -46,6 +47,7 @@ import (
 	"github.com/gohugoio/hugo/resources/page"
 	"github.com/gohugoio/hugo/resources/page/siteidentities"
 	"github.com/gohugoio/hugo/resources/postpub"
+	"github.com/gohugoio/hugo/resources/resource"
 
 	"github.com/spf13/afero"
 
@@ -275,18 +277,13 @@ func (h *HugoSites) assemble(ctx context.Context, l logg.LevelLogger, bcfg *Buil
 
 	changes := assembleChanges.Changes()
 
-	// Changes from the assemble step (e.g. lastMod, cascase) needs a re-calculation
+	// Changes from the assemble step (e.g. lastMod, cascade) needs a re-calculation
 	// of what needs to be re-built.
 	if len(changes) > 0 {
 		if err := h.resolveAndClearStateForIdentities(ctx, l, nil, changes); err != nil {
 			return err
 		}
 	}
-	h.renderFormats = output.Formats{}
-	for _, s := range h.Sites {
-		s.s.initRenderFormats()
-		h.renderFormats = append(h.renderFormats, s.renderFormats...)
-	}
 
 	for _, s := range assemblers {
 		if err := s.assemblePagesStep2(); err != nil {
@@ -296,9 +293,16 @@ func (h *HugoSites) assemble(ctx context.Context, l logg.LevelLogger, bcfg *Buil
 
 	h.renderFormats = output.Formats{}
 	for _, s := range h.Sites {
+		s.s.initRenderFormats()
 		h.renderFormats = append(h.renderFormats, s.renderFormats...)
 	}
 
+	for _, s := range assemblers {
+		if err := s.assemblePagesStepFinal(); err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -594,6 +598,10 @@ type pathChange struct {
 	isDir bool
 }
 
+func (p pathChange) isStructuralChange() bool {
+	return p.delete || p.isDir
+}
+
 // processPartial prepares the Sites' sources for a partial rebuild.
 func (h *HugoSites) processPartial(ctx context.Context, l logg.LevelLogger, config *BuildCfg, init func(config *BuildCfg) error, events []fsnotify.Event) error {
 	h.Log.Trace(logg.StringFunc(func() string {
@@ -608,7 +616,7 @@ func (h *HugoSites) processPartial(ctx context.Context, l logg.LevelLogger, conf
 
 	// For a list of events for the different OSes, see the test output in https://github.com/bep/fsnotifyeventlister/.
 	events = h.fileEventsFilter(events)
-	events = h.fileEventsTranslate(events)
+	events = h.fileEventsTrim(events)
 	eventInfos := h.fileEventsApplyInfo(events)
 
 	logger := h.Log
@@ -728,7 +736,7 @@ func (h *HugoSites) processPartial(ctx context.Context, l logg.LevelLogger, conf
 
 		case files.ComponentFolderLayouts:
 			tmplChanged = true
-			templatePath := pathInfo.TrimLeadingSlash().PathNoLang()
+			templatePath := pathInfo.Unnormalized().TrimLeadingSlash().PathNoLang()
 			if !h.Tmpl().HasTemplate(templatePath) {
 				tmplAdded = true
 			}
@@ -756,15 +764,45 @@ func (h *HugoSites) processPartial(ctx context.Context, l logg.LevelLogger, conf
 				}
 			}
 		case files.ComponentFolderAssets:
-			logger.Println("Asset changed", pathInfo.Path())
+			p := pathInfo.Path()
+			logger.Println("Asset changed", p)
+
+			var matches []any
+			var mu sync.Mutex
+
+			h.MemCache.ClearMatching(
+				func(k string, pm dynacache.PartitionManager) bool {
+					// Avoid going through everything.
+					return strings.HasPrefix(k, "/res")
+				},
+				func(k, v any) bool {
+					if strings.Contains(k.(string), p) {
+						mu.Lock()
+						defer mu.Unlock()
+						switch vv := v.(type) {
+						case resource.Resources:
+							// GetMatch/Match.
+							for _, r := range vv {
+								matches = append(matches, r)
+							}
+							return true
+						default:
+							matches = append(matches, vv)
+							return true
+
+						}
+					}
+					return false
+				})
 
 			var hasID bool
-			r, _ := h.ResourceSpec.ResourceCache.Get(context.Background(), dynacache.CleanKey(pathInfo.Base()))
-			identity.WalkIdentitiesShallow(r, func(level int, rid identity.Identity) bool {
-				hasID = true
-				changes = append(changes, rid)
-				return false
-			})
+			for _, r := range matches {
+				identity.WalkIdentitiesShallow(r, func(level int, rid identity.Identity) bool {
+					hasID = true
+					changes = append(changes, rid)
+					return false
+				})
+			}
 			if !hasID {
 				changes = append(changes, pathInfo)
 			}
@@ -920,6 +958,17 @@ func (h *HugoSites) processPartial(ctx context.Context, l logg.LevelLogger, conf
 	return nil
 }
 
+func (h *HugoSites) LogServerAddresses() {
+	if h.hugoInfo.IsMultihost() {
+		for _, s := range h.Sites {
+			h.Log.Printf("Web Server is available at %s (bind address %s) %s\n", s.conf.C.BaseURL, s.conf.C.ServerInterface, s.Language().Lang)
+		}
+	} else {
+		s := h.Sites[0]
+		h.Log.Printf("Web Server is available at %s (bind address %s)\n", s.conf.C.BaseURL, s.conf.C.ServerInterface)
+	}
+}
+
 func (h *HugoSites) processFull(ctx context.Context, l logg.LevelLogger, config BuildCfg) (err error) {
 	if err = h.processFiles(ctx, l, config); err != nil {
 		err = fmt.Errorf("readAndProcessContent: %w", err)
